/*
  RF69 radio-Chip on In-Circuit radino RF69 modules
  for more information: www.in-circuit.de or www.radino.cc

  Copyright (c) 2015 In-Circuit GmbH

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software
  and associated documentation files (the "Software"), to deal in the Software without restriction,
  including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef ARDUINO_ARCH_AVR
#include <Arduino.h>
#define ICRF69_PAKETSIZE 61
typedef struct __attribute__((packed))
{
  uint8_t len;
  uint8_t packetdata[ICRF69_PAKETSIZE];
  int16_t rssi;
} transfer_t;

class IC_RF69
{
  public:
    void Init() {}
    void Sleep() {}
    float setFrequency(float desiredFreq) {}
    unsigned long setDatarate(unsigned long desiredDatarate) {}
    unsigned char setPaTable(unsigned char newValue) {}
    void Transmit(const unsigned char *txBuf, unsigned char cnt) {}
    unsigned char ReadPacket(transfer_t *rxBuf) {}
};
#else

#ifndef __IC_RF69_h__
#define __IC_RF69_h__
#include <Arduino.h>

#include <RFM69.h>

//Pins and SPI setup
#define ICRF69_SS_PIN  8
#define ICRF69_IRQ_PIN 7  //within arduino:
#define ICRF69_IRQ_NUM 4  //INT6 -> int = 4

#define FREQUENCY   RF69_433MHZ
#define MODULEADDRESSES   0xFF
#define NETWORKID   0
#define KEY         "sampleEncryptKey"

#define ICRF69_PAKETSIZE RF69_MAX_DATA_LEN

typedef struct __attribute__((packed))
{
  uint8_t len;
  uint8_t packetdata[ICRF69_PAKETSIZE];
  int16_t rssi;
} transfer_t;

//Class
class IC_RF69 : public RFM69
{
  private:
    float m_freq;
    
  public:
    IC_RF69() : RFM69(ICRF69_SS_PIN, ICRF69_IRQ_PIN, false, ICRF69_IRQ_NUM)
    {}
    float setFrequency(float desiredFreq)
    {}
    unsigned long setDatarate(unsigned long desiredDatarate)
    {}
    unsigned char setPaTable(unsigned char newValue)
    {}
    
    void Transmit(const unsigned char *txBuf, unsigned char cnt)
    {
      RFM69::send(MODULEADDRESSES, txBuf, cnt);
    }
    
    void Init()
    {
      RFM69::initialize(FREQUENCY,MODULEADDRESSES,NETWORKID);
      RFM69::encrypt(KEY);
      //radio.writeReg(REG_BITRATEMSB, RF_BITRATEMSB_38400);
      RFM69::writeReg(0x03, 0x03);
      //radio.writeReg(REG_BITRATELSB, RF_BITRATELSB_38400);
      RFM69::writeReg(0x04, 0x41);
    }
    
    void Sleep()
    {
      RFM69::sleep();
    }
    
    //get RXFIFO content if any else start listening
    unsigned char ReadPacket(transfer_t *rxBuf)
    {
      if (!RFM69::receiveDone())
        return 0;
      rxBuf->len = RFM69::DATALEN;
      rxBuf->rssi = RFM69::RSSI;
      for (uint8_t i=0;i<RFM69::DATALEN;i++)
        rxBuf->packetdata[i] = DATA[i];
      return (1+RFM69::DATALEN);
    }
};

#endif	//__IC_RF69_h__

#endif